/*
  This file is part of the LifeV library
  Copyright (C) 2001,2002,2003,2004 EPFL, INRIA and Politechnico di Milano

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <life/lifefilters/readMesh3D.hpp>
# include <stdlib.h>
# include <stdio.h>

namespace LifeV
{
// here I should cut and paste the end of the readMesh3D.h
// but I have troubles when linking (see the comments in readMesh3D.h)
//----------------------------------------------------------------------
//
// Problem: the functions that follows should be in a readMesh3D.cc
// nevertheless, if I do that, I can compile the library, but I
// obtain several errors during the linking with a main.cc.
// JFG, 07/09/2002
//======================================================================

bool
readMppFileHead( std::ifstream & mystream, UInt & numVertices, UInt & numBVertices, UInt & numBFaces, UInt & numBEdges, UInt & numVolumes )
{
    unsigned done = 0;
    std::string line;
    Real x, y, z;
    int ity, ity_id;
    UInt p1, p2, p3;
    UInt i, ibc;
    //streampos start=mystream.tellg();

    while ( next_good_line( mystream, line ).good() )
    {
        if ( line.find( "odes" ) != std::string::npos )
        {
            std::string node_s = line.substr( line.find_last_of( ":" ) + 1 );
            numVertices = atoi( node_s );
            done++;
            numBVertices = 0;
            for ( i = 0;i < numVertices;i++ )
            {
                mystream >> x >> y >> z >> ity >> ibc;
                if ( ity != 3 )
                {
#ifndef OLDMPPFILE
                    mystream >> ibc;
#endif

                    numBVertices++;
                }
            }
        }


        if ( line.find( "iangular" ) != std::string::npos )
        {
            std::string node_s = line.substr( line.find_last_of( ":" ) + 1 );
            numBFaces = atoi( node_s );
            done++;
            for ( i = 0;i < numBFaces;i++ )
            {
#ifdef OLDMPPFILE
                mystream >> p1 >> p2 >> p3 >> ity >> ibc;
#else

                mystream >> p1 >> p2 >> p3 >> ity >> ity_id >> ibc;
#endif

            }
        }

        if ( line.find( "oundary" ) != std::string::npos )
        {
            std::string node_s = line.substr( line.find_last_of( ":" ) + 1 );
            numBEdges = atoi( node_s );
            for ( i = 0;i < numBEdges;i++ )
            {
#ifdef OLDMPPFILE
                mystream >> p1 >> p2 >> ity >> ibc;
#else

                mystream >> p1 >> p2 >> ity >> ity_id >> ibc;
#endif

            }
            done++;
        }
        if ( line.find( "etrahedral" ) != std::string::npos )
        {
            std::string node_s = line.substr( line.find_last_of( ":" ) + 1 );
            numVolumes = atoi( node_s );
            done++;
        }
    }
    return done == 4 ;
}



// ****************      INRIA mesh  readers   **********************************



int nextIntINRIAMeshField( std::string const & line, std::istream & mystream )
{
    // first control if line has something. If so use atoi (the version from util_string.h) to extract
    // the integer. Otherwise get if from the input stream
    for ( std::string::const_iterator is = line.begin(); is != line.end(); ++is )
        if ( *is != ' ' )
            return atoi( line );
    int dummy;
    mystream >> dummy;
    return dummy;
}

//


//! It Reads all basic info from INRIA MESH file so as to be able to properly dimension all arrays
bool
readINRIAMeshFileHead( std::ifstream & mystream,
                       UInt & numVertices,
                       UInt & numBVertices,
                       UInt & numBFaces,
                       UInt & numBEdges,
                       UInt & numVolumes,
                       UInt & numStoredFaces,
                       ReferenceShapes & shape,
                       InternalEntitySelector iSelect )
{
    unsigned done = 0;
    std::string line;
    Real x, y, z;
    UInt p1, p2, p3, p4, p5, p6, p7, p8;
    UInt i, ibc;

    int idummy;

    UInt numReadFaces = 0;
    numStoredFaces    = 0;

    //shape = NONE;
    std::vector<bool> isboundary;
    //streampos start=mystream.tellg();

    while ( next_good_line( mystream, line ).good() )
    {

        if ( line.find( "MeshVersionFormatted" ) != std::string:: npos )
        {
            idummy = nextIntINRIAMeshField( line.substr( line.find_last_of( "d" ) + 1 ), mystream );
            ASSERT_PRE0( idummy == 1, "I can read only formatted INRIA Mesh files, sorry" );
        }

        if ( line.find( "Dimension" ) != std::string:: npos )
        {
            idummy = nextIntINRIAMeshField( line.substr( line.find_last_of( "n" ) + 1 ), mystream );
            ASSERT_PRE0( idummy == 3, "I can read only 3D INRIA Mesh files, sorry" );
        }

        // I assume that internal vertices have their Ref value set to 0 (not clear from medit manual)
        if ( line.find( "Vertices" ) != std::string::npos )
        {
            numVertices = nextIntINRIAMeshField( line.substr( line.find_last_of( "s" ) + 1 ), mystream );
            done++;
            numBVertices = 0;
            isboundary.resize(numVertices,false);

            for ( i = 0;i < numVertices;++i )
            {
                mystream >> x >> y >> z >> ibc;
                if ( ! iSelect(EntityFlag(ibc))){
                    numBVertices++;
                    isboundary[i]=true;
                }
            }
        }

        if ( line.find( "Triangles" ) != std::string::npos )
        {
            ASSERT_PRE0( shape != HEXA, " Cannot have triangular faces in an HEXA INRIA  MESH" );
            shape = TETRA;
            numReadFaces=nextIntINRIAMeshField( line.substr( line.find_last_of( "s" ) + 1 ), mystream );
            numBFaces = 0;

            done++;
            for ( UInt k = 0; k < numReadFaces; k++ )
            {
                mystream >> p1 >> p2 >> p3 >> ibc;
                if(isboundary[p1-1]&&isboundary[p2-1]&&isboundary[p3-1]){
                    if (iSelect(EntityFlag(ibc))){
                        std::cerr<<"ATTENTION: Face "<<p1<<" "<<p2<<" "<<p3<<
                            " has all vertices on the boundary yet is marked as interior: "<<ibc<<std::endl;
                    }
                    ++numBFaces;
                } else {
                    if (!iSelect(EntityFlag(ibc))){
                        std::cerr<<"ATTENTION: Face "<<p1<<" "<<p2<<" "<<p3<<
                            " has vertices in the interior yet is marked as boundary: "<<ibc<<std::endl;
                    }
                }
            }
            numStoredFaces=numReadFaces;
        }


        if ( line.find( "Quadrilaterals" ) != std::string::npos )
        {
            ASSERT_PRE0( shape != TETRA, " Cannot have quad faces in an TETRA INRIA MESH" );
            shape = HEXA;
            numReadFaces= nextIntINRIAMeshField( line.substr( line.find_last_of( "s" ) + 1 ), mystream );
            done++;
            numBFaces = 0;
            for ( UInt k = 0;k < numReadFaces;k++ )
            {
                mystream >> p1 >> p2 >> p3 >> p4 >> ibc;
                if(isboundary[p1-1]&&isboundary[p2-1]&&isboundary[p3-1]&&isboundary[p4-1]
                   ){
                    if (iSelect(EntityFlag(ibc))){
                        std::cerr<<"ATTENTION: Face "<<p1<<" "<<p2<<" "<<p3<<" "<<p4<<
                            " has all vertices on the boundary yet is marked as interior: "<<
                            ibc<<std::endl;
                    }
                    ++numBFaces;
                }

            }
            numStoredFaces=numReadFaces;
        }
        // To cope with a mistake in INRIA Mesh files
        if ( line.find( "Tetrahedra" ) != std::string::npos )
        {
            ASSERT_PRE0( shape != HEXA, " Cannot have tetras  in a HEXA INRIA MESH" );
            shape = TETRA;
            numVolumes = nextIntINRIAMeshField( line.substr( line.find_last_of( "a" ) + 1 ), mystream );
            done++;
            for ( i = 0;i < numVolumes;i++ )
            {
                mystream >> p1 >> p2 >> p3 >> p4 >> ibc;
            }
        }

        if ( line.find( "Hexahedra" ) != std::string::npos )
        {
            ASSERT_PRE0( shape != TETRA, " Cannot have Hexahedra in a TETRA INRIA MESH" );
            shape = HEXA;
            numVolumes = nextIntINRIAMeshField( line.substr( line.find_last_of( "a" ) + 1 ), mystream );
            done++;
            for ( i = 0;i < numVolumes;i++ )
            {
                mystream >> p1 >> p2 >> p3 >> p4 >> p5 >> p6 >> p7 >> p8 >> ibc;
            }
        }
        // I assume we are storing only boundary edges
        if ( line.find( "Edges" ) != std::string::npos )
        {
            numBEdges = nextIntINRIAMeshField( line.substr( line.find_last_of( "a" ) + 1 ), mystream );
            done++;
            for ( i = 0;i < numBEdges;i++ )
            {
                mystream >> p1 >> p2 >> ibc;
            }
        }
    }
    return true ;
}
}
